// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS.

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file apame_wcs.sci

tic
clear
xdel(winsid())
if MSDOS then
  //unix_s("del /f wing.inp wing.log wing.res");
else
  unix_s("rm -f wing.inp wing.log wing.res");
end
getf('write_input.sci')
getf('root_tip.sci')
getf('meshing.sci')
getf('read_results.sci')
getf('vooren.sci')
getf('foil_ext.sci')
getf('aero_force.sci')
getf('plotting.sci')
funcprot(0)
stacksize('max')
clc
pi = 3.141592653589793;

// geometry
span = 5;               // wing span                    [m]
croot = 1;              // root chord length            [m]
ctip = 0.3;             // tip chord length             [m]
tipAL = -3*pi/180;      // tip angle of attack          [degrees]
lambda14 = 30*pi/180;   // wing sweep (1/4 wing)        [degrees]
mode = 1;               // 0=external file, 1=van de Vooren airfoil (internal)
if mode == 0
    r_foil = "66415_60.DAT";    // root airfoil data
    t_foil = "66415_60.DAT";    // tip airfoil data
elseif mode == 1
    thickness_r = 0.05;         // thickness of root airfoil
    trailing_angle_r = 15;      // trailing angle of root airfoil
    thickness_t = 0.03;         // thickness of tip airfoil
    trailing_angle_t = 12;      // trailing angle of tip airfoil
end

// solver parameters
method = 0;             // singularity method: 0=const sourc/doubl, 1=const doubl
farpoint = 1000;        // farpoint distance for trailing wake  [m]
err = 1E-07;            // error
farfieldfaktor = 5;     // "far field" distance in longer panel diagonal
collcalc = 1;           // collocation point calculation: 0=approximative, 1=accurate
velorder = 1;           // interpolation order: 1-first, 2-second
velometh = 1;           // interpolation method: 0-directional, 1-surface
Mp = 15;                // number of panels on halfspan
if mode == 1
    N = 60;             // number of panels on airfoil
end

// flow parameters
V = 27.778;                 // reference speed          [m/s]
AL = [0 2 4]*pi/180;        // angle of attack          [degrees]
ro = 1.225;                 // air density              [kg/m^3]
p_ref = 101325;             // atmospheric pressure     [Pa]
mach = 0;                   // prandtl-glauert correction: 0=no correction, *=Mach number
find_ac = 0;                // 1=find aerodynamic center, 0=use following values
origin = [0 0 0];           // reference point

// reference aerodynamic values
lambda = ctip/croot;
c = 2/3*croot*(1+lambda^2/(1+lambda));  // mean aerodynamic chord
Sw = (croot+ctip)/2*span;               // wing surface
AR = span^2/Sw;                         // aspect ratio

// free stream velocity components
Vx = V*cos(AL);
Vz = V*sin(AL);

// spanwise numbering of panels
M = Mp*2;
MM = Mp+1;
M1 = M+1;
M2 = M+2;
if mode == 1
    N1 = N+1;
    N2 = N+2;
end

// reading (calculating) airfoil data
if mode == 0
    [N,N1,N2,xf1,zf1] = foil_ext(r_foil);
    [N,N1,N2,xf2,zf2] = foil_ext(t_foil);
elseif mode == 1
    [xf1,zf1] = vooren(N,N1,N2,thickness_r,trailing_angle_r);
    [xf2,zf2] = vooren(N,N1,N2,thickness_t,trailing_angle_t);
end

// positioning root and tip airfoil
[xp1,zp1,xp2,zp2] = root_tip(xf1,zf1,xf2,zf2,croot,ctip,tipAL,span,lambda14,err);

// interpolating panels on wing (full cosine distribution)
[x,y,z] = meshing(M1,Mp,pi,M2,N1,span,xp1,zp1,xp2,zp2);

// writing input file for APAME solver
write_input(find_ac,origin,V,ro,p_ref,mach,AL,span,c,Sw,method,farpoint,err,farfieldfaktor,collcalc,velorder,velometh,N1,MM,x,y,z);
[time] = toc();
disp(time,'Preparing input file time in seconds:')

tic
// sending input file for calculation
// please put solver executable in current folder or create environment to
// apame solver and modify following lines accordingly
if MSDOS then
  unix_s("..\bin\apame22_32 wing")
else
  unix_s("ulimit -s unlimited;../bin/./apame22_64 wing")
end
[time] = toc();
disp(time,'APAME solver time in seconds:')

tic
// reading results file
[origin,S,CX,CZ,CM,cp,v,doublet,vx,vy,vz,p_stat,p_dyna,p_mano] = read_results(AL,N,M);

// calculating aerodynamic forces
aero_force(AL,CX,CZ,CM,AR);

// plotting distributions:
// cp = pressure
// v = velocity
// doublet = doublet strengths
// vx,vy,vz = velocity components
// p_stat = static pressure
// p_dyna = dynamic pressure
// p_mano = manometer pressure
if length(cp) > 3
  plotting(x,y,z,N,Mp,cp(:,:,:,1));
else
  plotting(x,y,z,N,Mp,cp(:,:,:));
end

[time] = toc();
disp(time,'Time for post-processing in seconds:')
