! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file grid.f90

subroutine grid(FF,S,&
    &u1,u2,u3,o1,o2,o3,n1,n2,n3,p1,p2,p3,&
    &cx,cy,cz,x1,y1,x2,y2,x3,y3,x4,y4,d1,d2,d3,d4,x,y,z,&
    &max_column,max_column1,max_row1,max_row2,wake,error,farfield,collcalc)

implicit none

integer i,j,ii,jj
integer, intent(in) :: max_row1,max_row2,max_column,max_column1,collcalc
real, intent(in) :: wake,error,farfield
real AA,BB,modc,ux,uy,uz,uu,px,py,pz,pp,cx1,cx2,cx3,cx4,cy1,cy2,cy3,cy4,cz1,cz2,cz3,cz4
real, dimension(3) :: A,B,C,norm
real, dimension(max_row1,max_column) :: dsum
real, dimension(max_row2,max_column1), intent(inout) :: x,y,z
real, dimension(max_row1,max_column), intent(out) :: FF,S,&
    &u1,u2,u3,o1,o2,o3,n1,n2,n3,p1,p2,p3,&
    &cx,cy,cz,x1,y1,x2,y2,x3,y3,x4,y4,d1,d2,d3,d4

! calculation of vortex trail coordinates
do i=1,max_column1
    x(max_row2,i) = x(max_row1,i)+wake
    y(max_row2,i) = y(max_row1,i)
    z(max_row2,i) = z(max_row1,i)
enddo

! calculation of panel surfaces and surface normals
do j=1,max_column
    do i=1,max_row1
        ii = i+1
        jj = j+1
        A = (/(x(ii,jj)-x(i,j)), (y(ii,jj)-y(i,j)), (z(ii,jj)-z(i,j))/)
        B = (/(x(ii,j)-x(i,jj)), (y(ii,j)-y(i,jj)), (z(ii,j)-z(i,jj))/)
        AA = sqrt((x(ii,jj)-x(i,j))**2 + (y(ii,jj)-y(i,j))**2 + (z(ii,jj)-z(i,j))**2)
        BB = sqrt((x(ii,j)-x(i,jj))**2 + (y(ii,j)-y(i,jj))**2 + (z(ii,j)-z(i,jj))**2)
        FF(i,j) = farfield*max(AA,BB)
        C(1) = A(2)*B(3)-A(3)*B(2)
        C(2) = A(3)*B(1)-A(1)*B(3)
        C(3) = A(1)*B(2)-A(2)*B(1)
        modc = sqrt(C(1)**2 + C(2)**2 + C(3)**2)
        S(i,j) = modc/2
        if (modc .lt. error) then
            n1(i,j) = 0
            n2(i,j) = 0
            n3(i,j) = 0
        else
            norm = -C/modc
            n1(i,j) = norm(1)
            n2(i,j) = norm(2)
            n3(i,j) = norm(3)
        endif
    enddo
enddo

! calculation of u(longitudinal), p(transversal) and o(perpendicular) vectors
do j=1,max_column
    do i=1,max_row1
        ii = i+1
        jj = j+1
        ux = ((x(ii,j)+x(ii,jj))-(x(i,j)+x(i,jj)))/2
        uy = ((y(ii,j)+y(ii,jj))-(y(i,j)+y(i,jj)))/2
        uz = ((z(ii,j)+z(ii,jj))-(z(i,j)+z(i,jj)))/2
        uu = sqrt(ux**2 + uy**2 + uz**2)
        u1(i,j) = ux/uu
        u2(i,j) = uy/uu
        u3(i,j) = uz/uu
        px = ((x(i,jj)+x(ii,jj))-(x(i,j)+x(ii,j)))/2
        py = ((y(i,jj)+y(ii,jj))-(y(i,j)+y(ii,j)))/2
        pz = ((z(i,jj)+z(ii,jj))-(z(i,j)+z(ii,j)))/2
        pp = sqrt(px**2 + py**2 + pz**2)
        p1(i,j) = px/pp
        p2(i,j) = py/pp
        p3(i,j) = pz/pp
        o1(i,j) = n2(i,j)*u3(i,j) - n3(i,j)*u2(i,j)
        o2(i,j) = u1(i,j)*n3(i,j) - n1(i,j)*u3(i,j)
        o3(i,j) = n1(i,j)*u2(i,j) - n2(i,j)*u1(i,j)
    enddo
enddo

if (collcalc .eq. 0) then
    ! calculation of colocation points
    do j=1,max_column
        do i=1,max_row1
            ii = i+1
            jj = j+1
            cx(i,j) = (x(i,j) + x(i,jj) + x(ii,j) + x(ii,jj))/4
            cy(i,j) = (y(i,j) + y(i,jj) + y(ii,j) + y(ii,jj))/4
            cz(i,j) = (z(i,j) + z(i,jj) + z(ii,j) + z(ii,jj))/4
        enddo
    enddo
    ! calculation of panel coordinates and panel side lengths
    do j=1,max_column
        do i=1,max_row1
            ii = i+1
            jj = j+1
            x1(i,j) = (x(i,j)-cx(i,j))*u1(i,j)+(y(i,j)-cy(i,j))*u2(i,j)+(z(i,j)-cz(i,j))*u3(i,j)
            y1(i,j) = (x(i,j)-cx(i,j))*o1(i,j)+(y(i,j)-cy(i,j))*o2(i,j)+(z(i,j)-cz(i,j))*o3(i,j)
            x2(i,j) = (x(ii,j)-cx(i,j))*u1(i,j)+(y(ii,j)-cy(i,j))*u2(i,j)+(z(ii,j)-cz(i,j))*u3(i,j)
            y2(i,j) = (x(ii,j)-cx(i,j))*o1(i,j)+(y(ii,j)-cy(i,j))*o2(i,j)+(z(ii,j)-cz(i,j))*o3(i,j)
            x3(i,j) = (x(ii,jj)-cx(i,j))*u1(i,j)+(y(ii,jj)-cy(i,j))*u2(i,j)+(z(ii,jj)-cz(i,j))*u3(i,j)
            y3(i,j) = (x(ii,jj)-cx(i,j))*o1(i,j)+(y(ii,jj)-cy(i,j))*o2(i,j)+(z(ii,jj)-cz(i,j))*o3(i,j)
            x4(i,j) = (x(i,jj)-cx(i,j))*u1(i,j)+(y(i,jj)-cy(i,j))*u2(i,j)+(z(i,jj)-cz(i,j))*u3(i,j)
            y4(i,j) = (x(i,jj)-cx(i,j))*o1(i,j)+(y(i,jj)-cy(i,j))*o2(i,j)+(z(i,jj)-cz(i,j))*o3(i,j)
            d1(i,j) = sqrt((x2(i,j)-x1(i,j))**2 + (y2(i,j)-y1(i,j))**2)
            d2(i,j) = sqrt((x3(i,j)-x2(i,j))**2 + (y3(i,j)-y2(i,j))**2)
            d3(i,j) = sqrt((x4(i,j)-x3(i,j))**2 + (y4(i,j)-y3(i,j))**2)
            d4(i,j) = sqrt((x1(i,j)-x4(i,j))**2 + (y1(i,j)-y4(i,j))**2)
        enddo
    enddo
else
    ! calculation of panel side lengths
    do j=1,max_column
        do i=1,max_row1
            ii = i+1
            jj = j+1
            d1(i,j) = sqrt((x(ii,j)-x(i,j))**2 + (y(ii,j)-y(i,j))**2 + (z(ii,j)-z(i,j))**2)
            d2(i,j) = sqrt((x(ii,jj)-x(ii,j))**2 + (y(ii,jj)-y(ii,j))**2 + (z(ii,jj)-z(ii,j))**2)
            d3(i,j) = sqrt((x(i,jj)-x(ii,jj))**2 + (y(i,jj)-y(ii,jj))**2 + (z(i,jj)-z(ii,jj))**2)
            d4(i,j) = sqrt((x(i,j)-x(i,jj))**2 + (y(i,j)-y(i,jj))**2 + (z(i,j)-z(i,jj))**2)
            dsum(i,j) = d1(i,j) + d2(i,j) + d3(i,j) + d4(i,j)
        enddo
    enddo
    ! calculation of colocation points
    do j=1,max_column
        do i=1,max_row1
            ii = i+1
            jj = j+1
            cx1 = (x(ii,j)+x(i,j))/2
            cx2 = (x(ii,jj)+x(ii,j))/2
            cx3 = (x(i,jj)+x(ii,jj))/2
            cx4 = (x(i,j)+x(i,jj))/2
            cy1 = (y(ii,j)+y(i,j))/2
            cy2 = (y(ii,jj)+y(ii,j))/2
            cy3 = (y(i,jj)+y(ii,jj))/2
            cy4 = (y(i,j)+y(i,jj))/2
            cz1 = (z(ii,j)+z(i,j))/2
            cz2 = (z(ii,jj)+z(ii,j))/2
            cz3 = (z(i,jj)+z(ii,jj))/2
            cz4 = (z(i,j)+z(i,jj))/2
            cx(i,j) = (cx1*d1(i,j) + cx2*d2(i,j) + cx3*d3(i,j) + cx4*d4(i,j))/dsum(i,j)
            cy(i,j) = (cy1*d1(i,j) + cy2*d2(i,j) + cy3*d3(i,j) + cy4*d4(i,j))/dsum(i,j)
            cz(i,j) = (cz1*d1(i,j) + cz2*d2(i,j) + cz3*d3(i,j) + cz4*d4(i,j))/dsum(i,j)
        enddo
    enddo
    ! calculation of panel coordinates
    do j=1,max_column
        do i=1,max_row1
            ii = i+1
            jj = j+1
            x1(i,j) = (x(i,j)-cx(i,j))*u1(i,j)+(y(i,j)-cy(i,j))*u2(i,j)+(z(i,j)-cz(i,j))*u3(i,j)
            y1(i,j) = (x(i,j)-cx(i,j))*o1(i,j)+(y(i,j)-cy(i,j))*o2(i,j)+(z(i,j)-cz(i,j))*o3(i,j)
            x2(i,j) = (x(ii,j)-cx(i,j))*u1(i,j)+(y(ii,j)-cy(i,j))*u2(i,j)+(z(ii,j)-cz(i,j))*u3(i,j)
            y2(i,j) = (x(ii,j)-cx(i,j))*o1(i,j)+(y(ii,j)-cy(i,j))*o2(i,j)+(z(ii,j)-cz(i,j))*o3(i,j)
            x3(i,j) = (x(ii,jj)-cx(i,j))*u1(i,j)+(y(ii,jj)-cy(i,j))*u2(i,j)+(z(ii,jj)-cz(i,j))*u3(i,j)
            y3(i,j) = (x(ii,jj)-cx(i,j))*o1(i,j)+(y(ii,jj)-cy(i,j))*o2(i,j)+(z(ii,jj)-cz(i,j))*o3(i,j)
            x4(i,j) = (x(i,jj)-cx(i,j))*u1(i,j)+(y(i,jj)-cy(i,j))*u2(i,j)+(z(i,jj)-cz(i,j))*u3(i,j)
            y4(i,j) = (x(i,jj)-cx(i,j))*o1(i,j)+(y(i,jj)-cy(i,j))*o2(i,j)+(z(i,jj)-cz(i,j))*o3(i,j)
        enddo
    enddo
endif

return
end subroutine
